"use strict";
/* eslint-disable @typescript-eslint/ban-types */
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTss = void 0;
const react_1 = require("react");
const makeStyles_1 = require("./makeStyles");
const cssAndCx_1 = require("./cssAndCx");
const assert_1 = require("./tools/assert");
const Object_fromEntries_1 = require("./tools/polyfills/Object.fromEntries");
const objectKeys_1 = require("./tools/objectKeys");
const typeGuard_1 = require("./tools/typeGuard");
const getDependencyArrayRef_1 = require("./tools/getDependencyArrayRef");
const mergeClasses_1 = require("./mergeClasses");
const isSSR_1 = require("./tools/isSSR");
function createTss(params) {
    const { useContext, usePlugin, cache: cacheProvidedAtInception } = params;
    const { useCache } = (0, makeStyles_1.createUseCache)({ cacheProvidedAtInception });
    const { useCssAndCx } = (0, cssAndCx_1.createUseCssAndCx)({ useCache });
    const usePluginDefault = ({ classes, cx, css }) => ({ classes, cx, css });
    const tss = createTss_internal({
        useContext,
        useCache,
        useCssAndCx,
        "usePlugin": usePlugin !== null && usePlugin !== void 0 ? usePlugin : usePluginDefault,
        "idForSSR": undefined,
        "name": undefined
    });
    return { tss };
}
exports.createTss = createTss;
let counter = 0;
function createTss_internal(params) {
    const { useContext, useCache, useCssAndCx, usePlugin, idForSSR, name } = params;
    return {
        "withParams": () => createTss_internal(Object.assign({}, params)),
        "withName": name => createTss_internal(Object.assign(Object.assign({}, params), { name })),
        "withNestedSelectors": ({ idForSSR } = {}) => {
            if ((0, isSSR_1.isSSR)() && idForSSR === undefined) {
                console.warn("Nested selector require an uniq id to be provided: .withNestedSelectors({ idForSSR: <uniqId> })");
            }
            return createTss_internal(Object.assign(Object.assign({}, params), { "idForSSR": idForSSR !== null && idForSSR !== void 0 ? idForSSR : `${counter++}` }));
        },
        "createUseStyles": (cssObjectByRuleNameOrGetCssObjectByRuleName) => {
            const getCssObjectByRuleName = typeof cssObjectByRuleNameOrGetCssObjectByRuleName ===
                "function"
                ? cssObjectByRuleNameOrGetCssObjectByRuleName
                : () => cssObjectByRuleNameOrGetCssObjectByRuleName;
            return function useStyles(params) {
                var _a, _b, _c;
                const _d = (params !== null && params !== void 0 ? params : {}), { classesOverrides } = _d, paramsAndPluginParams = __rest(_d, ["classesOverrides"]);
                const context = useContext();
                const { css, cx } = useCssAndCx();
                const cache = useCache();
                let classes = (0, react_1.useMemo)(() => {
                    const refClassesCache = {};
                    // @ts-expect-error: Type safety non achievable.
                    const cssObjectByRuleName = getCssObjectByRuleName(Object.assign(Object.assign(Object.assign({}, params), context), (idForSSR === undefined
                        ? {}
                        : {
                            "classes": typeof Proxy === "undefined"
                                ? {}
                                : new Proxy({}, {
                                    "get": (_target, propertyKey) => {
                                        if (typeof propertyKey ===
                                            "symbol") {
                                            (0, assert_1.assert)(false);
                                        }
                                        return (refClassesCache[propertyKey] = `${cache.key}-${idForSSR}${name !== undefined
                                            ? `-${name}`
                                            : ""}-${propertyKey}-ref`);
                                    }
                                })
                        })));
                    const classes = (0, Object_fromEntries_1.objectFromEntries)((0, objectKeys_1.objectKeys)(cssObjectByRuleName).map(ruleName => {
                        const cssObject = cssObjectByRuleName[ruleName];
                        if (!cssObject.label) {
                            cssObject.label = `${name !== undefined ? `${name}-` : ""}${ruleName}`;
                        }
                        return [
                            ruleName,
                            `${css(cssObject)}${(0, typeGuard_1.typeGuard)(ruleName, ruleName in refClassesCache)
                                ? ` ${refClassesCache[ruleName]}`
                                : ""}`
                        ];
                    }));
                    (0, objectKeys_1.objectKeys)(refClassesCache).forEach(ruleName => {
                        if (ruleName in classes) {
                            return;
                        }
                        classes[ruleName] =
                            refClassesCache[ruleName];
                    });
                    return classes;
                }, [
                    cache,
                    css,
                    cx,
                    (0, getDependencyArrayRef_1.getDependencyArrayRef)(params),
                    ...Object.values(context)
                ]);
                classes = (0, react_1.useMemo)(() => (0, mergeClasses_1.mergeClasses)(classes, classesOverrides, cx), [classes, (0, getDependencyArrayRef_1.getDependencyArrayRef)(classesOverrides), cx]);
                // @ts-expect-error: Type safety non achievable.
                const pluginResultWrap = usePlugin(Object.assign(Object.assign({ classes,
                    css,
                    cx,
                    idForSSR,
                    name }, context), paramsAndPluginParams));
                return Object.assign({ "classes": (_a = pluginResultWrap.classes) !== null && _a !== void 0 ? _a : classes, "css": (_b = pluginResultWrap.css) !== null && _b !== void 0 ? _b : css, "cx": (_c = pluginResultWrap.cx) !== null && _c !== void 0 ? _c : cx }, context);
            };
        }
    };
}
